/*
** Module   : XDocument
** Abstract : DOM XML Document resource type interface.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 EVL 20130130 Created initial version.
** 002 EVK 20130531 Adding support PUBLIC-ID, SYSTEM-ID attributes and
**                  INITIALIZE-DOCUMENT-TYPE,IMPORT-NODE methods.
*/

package com.goldencode.p2j.xml;

import com.goldencode.p2j.util.*;

/**
 * Interface describing X-Document specific features of the DOM XML objects.
 */
public interface XDocument
extends XEntity,
        XmlSchema,
        Encoding
{
   // Methods
   // Returning value placed into hChild
   /**
    * Creating the new node within the current Document object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNode(handle hChild, String name, String type);
   
   /**
    * Creating the new node within the current Document object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNode(handle hChild, character name, String type);
   
   /**
    * Creating the new node within the current Document object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNode(handle hChild, String name, character type);
   
   /**
    * Creating the new node within the current Document object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNode(handle hChild, character name, character type);

   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, String uri,
                                      String name, String type);
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, character uri,
                                      String name, String type);
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, String uri,
                                      character name, String type);
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, String uri,
                                      String name, character type);
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, String uri,
                                      character name, character type);
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, character uri,
                                      String name, character type);
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, character uri,
                                      character name, String type);
   
   /**
    * Creating the new node within the current Document object using different
    * approach based on the name space URI.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be created and
    *           returned on method call to use further.
    * @param    uri
    *           The URI namespace of the node to be created.
    * @param    name
    *           The name of the node to be created.
    * @param    type
    *           The type of the node to be created.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical createNodeNamespace(handle hChild, character uri,
                                      character name, character type);
   
   // Returning value placed into hRoot
   /**
    * Getting the root node for the current document. The handle is placed
    * into passed object reference.
    * 
    * @param    hRoot
    *           The valid handle class instance variable to receive the
    *           result.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical getDocumentElement(handle hRoot);
   
   /**
    * Loading X-Document form file or memptr sources.
    * 
    * @param    mode
    *           Variable indicating the type of source to read. "file" means
    *           reading from file, "memptr" means reading from memory buffer.
    * @param    source
    *           If mode == "file" this is the file name, if "memptr" this is
    *           the memptr object.
    * @param    validate
    *           Flag indicating whether to validate while reading or not.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical load(String mode, Object source, logical validate);
   
   /**
    * Loading X-Document form file or memptr sources.
    * 
    * @param    mode
    *           Variable indicating the type of source to read. "file" means
    *           reading from file, "memptr" means reading from memory buffer.
    * @param    source
    *           If mode == "file" this is the file name, if "memptr" this is
    *           the memptr object.
    * @param    validate
    *           Flag indicating whether to validate while reading or not.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical load(String mode, Object source, boolean validate);
   
   /**
    * Loading X-Document form file or memptr sources.
    * 
    * @param    mode
    *           Variable indicating the type of source to read. "file" means
    *           reading from file, "memptr" means reading from memory buffer.
    * @param    source
    *           If mode == "file" this is the file name, if "memptr" this is
    *           the memptr object.
    * @param    validate
    *           Flag indicating whether to validate while reading or not.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical load(character mode, Object source, boolean validate);
   
   /**
    * Loading X-Document form file or memptr sources.
    * 
    * @param    mode
    *           Variable indicating the type of source to read. "file" means
    *           reading from file, "memptr" means reading from memory buffer.
    * @param    source
    *           If mode == "file" this is the file name, if "memptr" this is
    *           the memptr object.
    * @param    validate
    *           Flag indicating whether to validate while reading or not.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical load(character mode, Object source, logical validate);
   
   /**
    * Saving X-Document to file or stream or memptr object.
    * 
    * @param    mode
    *           Variable indicating the type of target to write. "file" means
    *           writing to file, "stream" means writing to the stream and
    *           "memptr" means writing to memory buffer.
    * @param    target
    *           If mode == "file" this is the file name, if "stream" means
    *           output stream, if "memptr" this is the memptr object.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical save(String mode, Object target);

   /**
    * Saving X-Document to file or stream or memptr object.
    * 
    * @param    mode
    *           Variable indicating the type of target to write. "file" means
    *           writing to file, "stream" means writing to the stream and
    *           "memptr" means writing to memory buffer.
    * @param    target
    *           If mode == "file" this is the file name, if "stream" means
    *           output stream, if "memptr" this is the memptr object.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical save(character mode, Object target);

   /**
    * Creates a new XML document, initializes the document based on the referenced DTD,
    * and creates its root node.
    *
    * @param    namespaceUri
    *           character expression representing the namespace Uniform Resource Identifier
    *           (URI) you want associated with the root node of the XML document. The
    *           namespace-uri must be unique and persistent.
    * @param    rootNodeName
    *           A character expression representing the name of the root node as defined in the
    *           XML document. If you are using namespaces and you want to associate a prefix
    *           with the namespace, you must qualify this node name with the namespace-uri
    *           and a colon character prefix (for example, namespace-uri:root-node-name).
    *           You must explicitly set the xmlns attribute on the root node.
    * @param    publicId
    *           An optional character expression representing the public ID of the DTD.
    * @param    systemId
    *           A required character expression representing the system ID of the DTD. This
    *           contains the path to the DTD which is either a file system path or an HTTP URL.
    *
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical initializeDocumentType(character namespaceUri,
                                         character rootNodeName,
                                         character publicId,
                                         character systemId);

   /**
    * Creates a new XML document, initializes the document based on the referenced DTD,
    * and creates its root node.
    *
    * @param    namespaceUri
    *           character expression representing the namespace Uniform Resource Identifier
    *           (URI) you want associated with the root node of the XML document. The
    *           namespace-uri must be unique and persistent.
    * @param    rootNodeName
    *           A character expression representing the name of the root node as defined in the
    *           XML document. If you are using namespaces and you want to associate a prefix
    *           with the namespace, you must qualify this node name with the namespace-uri
    *           and a colon character prefix (for example, namespace-uri:root-node-name).
    *           You must explicitly set the xmlns attribute on the root node.
    * @param    publicId
    *           An optional character expression representing the public ID of the DTD.
    * @param    systemId
    *           A required character expression representing the system ID of the DTD. This
    *           contains the path to the DTD which is either a file system path or an HTTP URL.
    *
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical initializeDocumentType(character namespaceUri,
                                         String    rootNodeName,
                                         character publicId,
                                         character systemId);

   /**
    * Creates a new XML document, initializes the document based on the referenced DTD,
    * and creates its root node.
    *
    * @param    namespaceUri
    *           character expression representing the namespace Uniform Resource Identifier
    *           (URI) you want associated with the root node of the XML document. The
    *           namespace-uri must be unique and persistent.
    * @param    rootNodeName
    *           A character expression representing the name of the root node as defined in the
    *           XML document. If you are using namespaces and you want to associate a prefix
    *           with the namespace, you must qualify this node name with the namespace-uri
    *           and a colon character prefix (for example, namespace-uri:root-node-name).
    *           You must explicitly set the xmlns attribute on the root node.
    * @param    publicId
    *           An optional character expression representing the public ID of the DTD.
    * @param    systemId
    *           A required character expression representing the system ID of the DTD. This
    *           contains the path to the DTD which is either a file system path or an HTTP URL.
    *
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical initializeDocumentType(String    namespaceUri,
                                         character rootNodeName,
                                         String    publicId,
                                         String    systemId);

   /**
    * This method returns the public ID of the external DTD from which an XML document
    * was generated.
    *
    * @return   public-id of DTD
    */
   public character getPublicId();

   /**
    * Returns the system ID of the external DTD from which an XML document was
    * generated. This contains the path to the DTD which is either a file system path or an
    * HTTP URL. The ABL parser uses this information to retrieve the DTD when parsing the
    * document.
    *
    * @return   system-id of DTD
    */
   public character getSystemId();

   /**
    * Creating the new node within the current Document object.
    *
    * @param    xNodeRef
    *           The valid handle class instance to use for the new XML node.
    * @param    xSourceNodeRef
    *           The valid handle class instance to import from.
    * @param    deep
    *           A logical that if <code>true</code> specifies that the whole sub-tree is to be
    *           copied. The default value is {@code false}.
    *
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical importNode(handle xNodeRef, handle xSourceNodeRef, logical deep);
}
